hammer_name = 'tg-hammer'
hammer_description = 'Helpers for fabric based deployments.'
hammer_version = '0.0.3'

__name__ = hammer_name
__description__ = hammer_description
__version__ = hammer_version
